pub type OptionGroup64<T> = OptionGroup<u64, T>;Expand description
A group of up to 64 Options with the discriminants packed into a single u64.
Aliased Type
pub struct OptionGroup64<T> { /* private fields */ }pub type OptionGroup64<T> = OptionGroup<u64, T>;A group of up to 64 Options with the discriminants packed into a single u64.
pub struct OptionGroup64<T> { /* private fields */ }