Traits abstracting over storage strategies and index types.
Structs
- Alloc
Storage alloc - A fat pointer to a heap-allocated storage block conforming to a
LayoutSpec. - Arena
Storage - A fat pointer to an arena-allocated storage block conforming to a
LayoutSpec. - Array
Layout - Inconstructible type to mark data structures that require an array-like storage layout.
Traits
- Capacity
- Types that can be used for indexing into array-like data structures.
- Layout
Spec - Types that specify a data structure’s storage layout requirements.
- Storage
- An interface to a contiguous memory block for use by data structures.
Type Aliases
- Inline
Storage - Shorthand for
[MaybeUninit<T>; C]for use with generic data structures. - Slice
Storage - Shorthand for
&'a mut [MaybeUninit<T>]for use with generic data structures.